unit      NumInput;

{ ************************************************************************** }
{ * Numeric Input for Turbo Vision                                         * }
{ * ------------------------------                                         * }
{ *                                                                        * }
{ * Offers you the possibility of a Number-, Byte- and Word-inputlines.    * }
{ *                                                                        * }
{ * Byte- and Word-inputlines are range-checked at the moment the data     * }
{ * is entered, so no validation-checking afterwards is needed.            * }
{ *                                                                        * }
{ * Warning: Let the length of your inputview be long enough, because      * }
{ *          the scrolling of this unit is not that good. (In my           * }
{ *          opinion not necessary)                                        * }
{ *                                                                        * }
{ * Author: Edwin Groothuis (S89405079@hse.nl or 2:284/205.1@fidonet)      * }
{ * How to get it: internet-users:                                         * }
{ *                 garbo.uwasa.fi:/pc/turbovis/numinp1.zip                * }
{ *                 oak.oakland.edu:/pub/msdos/turbovis/numinp1.zip        * }
{ *                fidonet-users:                                          * }
{ *                 freq NUMINP at 2:284/205 (+31-40-550352)               * }
{ *                 download NUMINP1.ZIP from 2:284/205 area: TurboPascal  * }
{ *                                                                        * }
{ * The source is hereby Public Domain, no warrenties are made, feel free  * }
{ * to modify or update. Please send bugs reports/fixes to me.             * }
{ *                                                                        * }
{ ************************************************************************** }

{$F+}
interface

uses      dialogs,drivers,objects;

type
{ A NumInputLine gives you a string with the length specified in Init
  with only the characters: 0,1,2,3,4,5,6,7,8,9,.,+,-. }
          PNumInputLine=^TNumInputLine;
          TNumInputLine=object(TInputLine)
                          procedure HandleEvent(var E:TEvent);virtual;
                        end;
{ A ByteInputLine gives you a byte. Can be in the range of 0..255 }
          PByteInputLine=^TByteInputLine;
          TByteInputLine=object(TInputLine)
                           constructor Init(var Bounds:TRect);
                           procedure HandleEvent(var E:TEvent);virtual;
                           function  DataSize:Word;virtual;
                           procedure GetData(var Rec);virtual;
                           procedure SetData(var Rec);virtual;
                        end;
{ A WordInputLine gives you a word. Can be in the range of 0..65535 }
          PWordInputLine=^TWordInputLine;
          TWordInputLine=object(TInputLine)
                           constructor Init(var Bounds:TRect);
                           procedure HandleEvent(var E:TEvent);virtual;
                           function  DataSize:Word;virtual;
                           procedure GetData(var Rec);virtual;
                           procedure SetData(var Rec);virtual;
                        end;

implementation


procedure TNumInputLine.HandleEvent(var E:TEvent);
begin
  if E.What=evKeyDown then
  begin
    if (E.CharCode in ['0'..'9','-','+','.']) and (SelEnd<>0) then
    begin
      SelectAll(FALSE);
      Data^:='';
    end;
    case E.CharCode of
      '0'..'9':
           if length(Data^)<MaxLen then
           begin
             Insert(E.CharCode,Data^,CurPos+1);
             Inc(CurPos);
           end;
      '-': if Data^<>'' then
             if Data^[1]='-' then
               system.delete(Data^,1,1)
             else
               if length(Data^)<MaxLen then
                 system.insert('-',Data^,1);
      '+': if Data^<>'' then
             if Data^[1]='-' then
               system.delete(Data^,1,1);
      '.': if Data^='' then
             Data^:='0.'
           else
             if length(Data^)<MaxLen then
               if pos('.',Data^)=0 then
               begin
                 if CurPos<>0 then
                   system.insert('.',Data^,CurPos+1)
                 else if length(Data^)<MaxLen-1 then
                 begin
                   system.insert('0.',Data^,1);
                   Inc(CurPos);
                 end else Dec(CurPos);
                 Inc(CurPos);
               end;
      #8 : begin
             if CurPos<>0 then
             begin
               system.delete(Data^,CurPos,1);
               Dec(CurPos);
               SelEnd:=0;
             end;
           end;
      #0: case E.KeyCode of
            kbLeft:
                 If CurPos<>0 then
                 begin
                   Dec(CurPos);
                   SelEnd:=0;
                 end;
            kbRight:
                 If CurPos<>Length(Data^) then
                 begin
                   Inc(CurPos);
                   SelEnd:=0;
                 end;
            kbHome:
                 begin
                   CurPos:=0;
                   SelEnd:=0;
                 end;
            kbEnd:
                 begin
                   CurPos:=Length(Data^);
                   SelEnd:=0;
                 end;
            kbDel:
                 begin
                   if SelEnd=0 then
                   begin
                     system.delete(Data^,CurPos+1,1);
                   end else begin
                     Data^:='';
                     CurPos:=0;
                   end;
                   SelEnd:=0;
                 end;
            else ;
          end;
    end;
  end;
  inherited Draw;
end;


{****************************************************************************}


constructor TByteInputLine.Init(var Bounds:TRect);
begin
  Inherited Init(Bounds,3);
end;



procedure TByteInputLine.HandleEvent(var E:TEvent);
begin
  if E.What=evKeyDown then
  begin
    if (E.CharCode in ['0'..'9']) and (SelEnd<>0) then
    begin
      SelectAll(FALSE);
      Data^:='';
    end;
    case E.CharCode of
      '0'..'9':
           if length(Data^)<MaxLen then
           begin
             case length(Data^) of
               0: begin
                    Insert(E.CharCode,Data^,CurPos+1);
                    Inc(CurPos);
                  end;
               1: if CurPos<>0 then
                  begin
                    if (Data^='0') then
                    begin
                      if (E.CharCode<>'0') then
                        Data^:=E.CharCode;
                    end else begin
                      Insert(E.CharCode,Data^,CurPos+1);
                      Inc(CurPos);
                    end;
                  end else
                    if E.CharCode<>'0' then
                    begin
                      Insert(E.CharCode,Data^,CurPos+1);
                      Inc(CurPos);
                    end;
               2: begin
                    case E.CharCode of
                      '0':      if CurPos<>0 then
                                begin
                                  if Data^<='25' then
                                  begin
                                    Insert('0',Data^,CurPos+1);
                                    Inc(CurPos);
                                  end;
                                end;
                      '1':      if CurPos<>0 then
                                begin
                                  if Data^<='25' then
                                  begin
                                    Insert('1',Data^,CurPos+1);
                                    Inc(CurPos);
                                  end;
                                end else begin
                                  Insert('1',Data^,1);
                                  Inc(CurPos);
                                end;
                      '2':      if CurPos<>0 then
                                begin
                                  if Data^<='25' then
                                  begin
                                    Insert('2',Data^,CurPos+1);
                                    Inc(CurPos);
                                  end;
                                end else begin
                                  if Data^<='55' then
                                  begin
                                    Insert('2',Data^,1);
                                    Inc(CurPos);
                                  end;
                                end;
                      '3'..'4': if CurPos<>0 then
                                begin
                                  if Data^<='25' then
                                  begin
                                    Insert(E.CharCode,Data^,CurPos+1);
                                    Inc(CurPos);
                                  end;
                                end;
                      '5':      if CurPos<>0 then
                                begin
                                  if Data^<='25' then
                                  begin
                                    Insert('5',Data^,CurPos+1);
                                    Inc(CurPos);
                                  end;
                                end;
                      '6'..'9': case CurPos of
                                  0: ;
                                  1: if Data^<='19' then
                                     begin
                                       Insert(E.CharCode,Data^,2);
                                       Inc(CurPos);
                                     end;
                                  2: if Data^<='24' then
                                     begin
                                       Insert(E.CharCode,Data^,3);
                                       Inc(CurPos);
                                     end;
                                end;
                    end;
                  end;
             end;
           end;
      #8 : begin
             if CurPos<>0 then
             begin
               system.delete(Data^,CurPos,1);
               Dec(CurPos);
               SelEnd:=0;
             end;
           end;
      #0: case E.KeyCode of
            kbLeft:
                 If CurPos<>0 then
                 begin
                   Dec(CurPos);
                   SelEnd:=0;
                 end;
            kbRight:
                 If CurPos<>Length(Data^) then
                 begin
                   Inc(CurPos);
                   SelEnd:=0;
                 end;
            kbHome:
                 begin
                   CurPos:=0;
                   SelEnd:=0;
                 end;
            kbEnd:
                 begin
                   CurPos:=Length(Data^);
                   SelEnd:=0;
                 end;
            kbDel:
                 begin
                   if SelEnd=0 then
                   begin
                     system.delete(Data^,CurPos+1,1);
                   end else begin
                     Data^:='';
                     CurPos:=0;
                   end;
                   SelEnd:=0;
                 end;
            else ;
          end;
    end;
  end;
  inherited Draw;
end;



function  TByteInputLine.DataSize:Word;
begin
  DataSize:=1;
end;
procedure TByteInputLine.SetData(var Rec);
begin
  str(Byte(Rec),Data^);
end;
procedure TByteInputLine.GetData(var Rec);
var       code:integer;
begin
  val(Data^,Byte(Rec),code);
end;


{****************************************************************************}


constructor TWordInputLine.Init(var Bounds:TRect);
begin
  Inherited Init(Bounds,5);
end;



procedure TWordInputLine.HandleEvent(var E:TEvent);
begin
  if E.What=evKeyDown then
  begin
    if (E.CharCode in ['0'..'9']) and (SelEnd<>0) then
    begin
      SelectAll(FALSE);
      Data^:='';
    end;
    case E.CharCode of
      '0'..'9':
           if length(Data^)<MaxLen then
           begin
             case length(Data^) of
               0: begin
                    Insert(E.CharCode,Data^,CurPos+1);
                    Inc(CurPos);
                  end;
               1: if CurPos<>0 then
                  begin
                    if (Data^='0') then
                    begin
                      if (E.CharCode<>'0') then
                        Data^:=E.CharCode;
                    end else begin
                      Insert(E.CharCode,Data^,CurPos+1);
                      Inc(CurPos);
                    end;
                  end else
                    if E.CharCode<>'0' then
                    begin
                      Insert(E.CharCode,Data^,CurPos+1);
                      Inc(CurPos);
                    end;
               2,3: if CurPos<>0 then
                  begin
                    Insert(E.CharCode,Data^,CurPos+1);
                    Inc(CurPos);
                  end else
                    if E.CharCode<>'0' then
                    begin
                      Insert(E.CharCode,Data^,CurPos+1);
                      Inc(CurPos);
                    end;
               4: begin
                    case E.CharCode of
                      '0': case CurPos of
                             0: ;
                             1,2: begin
                                  if Data^<='6535' then
                                  begin
                                    Insert('0',Data^,CurPos+1);
                                    Inc(CurPos);
                                  end;
                                end;
                             3: begin
                                  if Data^<='6555' then
                                  begin
                                    Insert('0',Data^,4);
                                    Inc(CurPos);
                                  end;
                                end;
                             4: begin
                                  if Data^<='6553' then
                                  begin
                                    Insert('0',Data^,5);
                                    Inc(CurPos);
                                  end;
                                end;
                           end;
                      '1','2':
                             case CurPos of
                             0: begin
                                  Insert(E.CharCode,Data^,1);
                                  Inc(CurPos);
                                end;
                             1: if Data^<='6999' then
                                begin
                                  Insert(E.CharCode,Data^,2);
                                  Inc(CurPos);
                                end;
                             2: if Data^<='6599' then
                                begin
                                  Insert(E.CharCode,Data^,3);
                                  Inc(CurPos);
                                end;
                             3: if Data^<='6559' then
                                begin
                                  Insert(E.CharCode,Data^,4);
                                  Inc(CurPos);
                                end;
                             4: if Data^<='6553' then
                                begin
                                  Insert(E.CharCode,Data^,5);
                                  Inc(CurPos);
                                end;
                           end;
                      '3': case CurPos of
                             0: begin
                                  Insert('3',Data^,1);
                                  Inc(CurPos);
                                end;
                             1: if Data^<='6999' then
                                begin
                                  Insert('3',Data^,2);
                                  Inc(CurPos);
                                end;
                             2: if Data^<='6599' then
                                begin
                                  Insert('3',Data^,3);
                                  Inc(CurPos);
                                end;
                             3: if Data^<='6555' then
                                begin
                                  Insert('3',Data^,4);
                                  Inc(CurPos);
                                end;
                             4: if Data^<='6553' then
                                begin
                                  Insert('3',Data^,5);
                                  Inc(CurPos);
                                end;
                           end;
                      '4': case CurPos of
                             0: begin
                                  Insert('4',Data^,1);
                                  Inc(CurPos);
                                end;
                             1: if Data^<='6999' then
                                begin
                                  Insert('4',Data^,2);
                                  Inc(CurPos);
                                end;
                             2: if Data^<='6599' then
                                begin
                                  Insert('4',Data^,3);
                                  Inc(CurPos);
                                end;
                             3: if Data^<='6549' then
                                begin
                                  Insert('4',Data^,4);
                                  Inc(CurPos);
                                end;
                             4: if Data^<='6553' then
                                begin
                                  Insert('4',Data^,5);
                                  Inc(CurPos);
                                end;
                           end;
                      '5': case CurPos of
                             0: begin
                                  Insert('5',Data^,1);
                                  Inc(CurPos);
                                end;
                             1,2: if Data^<='6535' then
                                begin
                                  Insert('5',Data^,CurPos+1);
                                  Inc(CurPos);
                                end;
                             3: if Data^<='6549' then
                                begin
                                  Insert('5',Data^,4);
                                  Inc(CurPos);
                                end;
                             4: if Data^<='6553' then
                                begin
                                  Insert('5',Data^,5);
                                  Inc(CurPos);
                                end;
                           end;
                      '6': case CurPos of
                             0: if Data^<='5535' then
                                begin
                                  Insert('6',Data^,1);
                                  Inc(CurPos);
                                end;
                             1: if data^<='5999' then
                                begin
                                  Insert('6',Data^,2);
                                  Inc(CurPos);
                                end;
                             2: if Data^<='6499' then
                                begin
                                  Insert('6',Data^,3);
                                  Inc(CurPos);
                                end;
                             3: if Data^<='6549' then
                                begin
                                  Insert('6',Data^,4);
                                  Inc(CurPos);
                                end;
                             4: if Data^<='6552' then
                                begin
                                  Insert('6',Data^,5);
                                  Inc(CurPos);
                                end;
                           end;
                      '7'..'9':
                           case CurPos of
                             0: ;
                             1: if data^<='5999' then
                                begin
                                  Insert('7',Data^,2);
                                  Inc(CurPos);
                                end;
                             2: if Data^<='6499' then
                                begin
                                  Insert('7',Data^,3);
                                  Inc(CurPos);
                                end;
                             3: if Data^<='6549' then
                                begin
                                  Insert('7',Data^,4);
                                  Inc(CurPos);
                                end;
                             4: if Data^<='6552' then
                                begin
                                  Insert('7',Data^,5);
                                  Inc(CurPos);
                                end;
                           end;
                    end;
                  end;
             end;
           end;
      #8 : begin
             if CurPos<>0 then
             begin
               system.delete(Data^,CurPos,1);
               Dec(CurPos);
               SelEnd:=0;
             end;
           end;
      #0: case E.KeyCode of
            kbLeft:
                 If CurPos<>0 then
                 begin
                   Dec(CurPos);
                   SelEnd:=0;
                 end;
            kbRight:
                 If CurPos<>Length(Data^) then
                 begin
                   Inc(CurPos);
                   SelEnd:=0;
                 end;
            kbHome:
                 begin
                   CurPos:=0;
                   SelEnd:=0;
                 end;
            kbEnd:
                 begin
                   CurPos:=Length(Data^);
                   SelEnd:=0;
                 end;
            kbDel:
                 begin
                   if SelEnd=0 then
                   begin
                     system.delete(Data^,CurPos+1,1);
                   end else begin
                     Data^:='';
                     CurPos:=0;
                   end;
                   SelEnd:=0;
                 end;
            else ;
          end;
    end;
  end;
  inherited Draw;
end;



function  TWordInputLine.DataSize:Word;
begin
  DataSize:=2;
end;
procedure TWordInputLine.SetData(var Rec);
begin
  str(Word(Rec),Data^);
end;
procedure TWordInputLine.GetData(var Rec);
var       code:integer;
begin
  val(Data^,Word(Rec),code);
end;



end.
{$F-}
